@@ -119,6 +119,7 @@ struct mount {
 	NihHash *           physical_dev_ids;
 	int                 physical_dev_ids_needed;
 	pid_t               fsck_pid;
+	int                 fsck_progress;
 	int                 fsck_fix;
 	int                 ready;
 
@@ -442,6 +443,7 @@ new_mount (const char *mountpoint,
 	mnt->physical_dev_ids = NULL;
 	mnt->physical_dev_ids_needed = TRUE;
 	mnt->fsck_pid = -1;
+	mnt->fsck_progress = -1;
 	mnt->fsck_fix = FALSE;
 	mnt->ready = FALSE;
 
@@ -2016,6 +2018,7 @@ run_fsck (Mount *mnt)
 
 	mnt->error = ERROR_NONE;
 	mnt->fsck_pid = spawn (mnt, args, FALSE, run_fsck_finished);
+	mnt->fsck_progress = -1;
 
 	fsck_update ();
 
@@ -2041,6 +2044,7 @@ run_fsck_finished (Mount *mnt,
 	/* The check is done */
 	mnt->error = ERROR_NONE;
 	mnt->fsck_pid = -1;
+	mnt->fsck_progress = -1;
 
  	fsck_update ();
 
@@ -2741,7 +2745,10 @@ fsck_reader (Mount *     mnt,
 			nih_assert_not_reached ();
 		}
 
-		plymouth_progress (mnt, progress);
+		if (mnt->fsck_progress != progress) {
+			plymouth_progress (mnt, progress);
+			mnt->fsck_progress = progress;
+		}
 	}
 }
 
@@ -3033,6 +3040,8 @@ plymouth_answer (void *             user_data,
 	case 'F':
 		if (mnt->error != ERROR_FSCK_FAILED)
 			break;
+		if (plymouth_error == ERROR_FSCK_IN_PROGRESS)
+			break;
 
 		mnt->error = ERROR_NONE;
 		plymouth_update (FALSE);
@@ -3046,7 +3055,9 @@ plymouth_answer (void *             user_data,
 	case 'i':
 	case 'I':
 		if ((mnt->error != ERROR_FSCK_FAILED)
-		    && (mnt->error == ERROR_FSCK_FAILED_HARD))
+		    && (mnt->error != ERROR_FSCK_FAILED_HARD))
+			break;
+		if (plymouth_error == ERROR_FSCK_IN_PROGRESS)
 			break;
 
 		mnt->error = ERROR_NONE;
@@ -3060,7 +3071,7 @@ plymouth_answer (void *             user_data,
 		break;
 	case 's':
 	case 'S':
-		if (mnt->error == ERROR_FSCK_IN_PROGRESS)
+		if (plymouth_error == ERROR_FSCK_IN_PROGRESS)
 			break;
 
 		mnt->error = ERROR_NONE;
@@ -3073,7 +3084,7 @@ plymouth_answer (void *             user_data,
 		break;
 	case 'm':
 	case 'M':
-		if (mnt->error == ERROR_FSCK_IN_PROGRESS)
+		if (plymouth_error == ERROR_FSCK_IN_PROGRESS)
 			break;
 
 		mnt->error = ERROR_NONE;
@@ -3104,10 +3115,9 @@ plymouth_answer (void *             user_data,
 		NIH_LIST_FOREACH (mounts, iter) {
 			Mount *mnt = (Mount *)iter;
 
-			if (mnt->fsck_pid > 0)
+			if ((mnt->fsck_pid > 0)
+			    && (mnt->fsck_progress >= 0))
 				kill (mnt->fsck_pid, SIGTERM);
-			if (mnt->error == ERROR_FSCK_IN_PROGRESS)
-				mnt->error = ERROR_NONE;
 		}
 		break;
 	default:
